import glob, os, datetime, secrets
from imp import load_source
from application.settings import application_settings
from libraries.system.crypt import Crypt
from libraries.system.migration.model import MigrationModel
from libraries.system.migration.manager import MigrationManager
from profilelib.libraries.api.model import ApiKeyModel
from profilelib.libraries.user.model import UserModel
from profilelib.libraries.role.model import RoleModel, UserRoleModel


class ProfilLibMigrationManager(MigrationManager):
    def __init__(self):
        self.current_file = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

    def _get_defined_migrations(self):
        """ Returns list of all the migrations that are defined in the application """
        migration_dir = self.current_file+'/migrations'

        defined_migration_list = glob.glob(migration_dir + '/[!_]*.*')
        # print(defined_migration_list)
        defined_migration_dict = {}
        for defined_migration in defined_migration_list:
            migration_name = os.path.basename(defined_migration)
            migration_name = migration_name.replace(".py", "")
            defined_migration_dict[migration_name] = defined_migration

        return defined_migration_dict

    def run(self):
        MigrationModel.create_table(True)
        self._set_pending_migrations()
        self._migrate()

    def _migrate(self):
        total_pending_migrations = len(self._pending_migrations.keys())
        if total_pending_migrations > 0:
            print("%s new migration(s) collected." % total_pending_migrations)
            for migration_name, migration in self._pending_migrations.items():
                try:
                    migration_module = load_source('Migration', migration)
                    migration_module = migration_module.Migration()
                    getattr(migration_module, self.get_migration_action())()
                    print(
                        "migration script %s:%s executed successfully." % (migration_name, self.get_migration_action()))
                except Exception as e:
                    print(e)
                else:
                    if self.get_migration_action() == 'up':
                        MigrationModel.create(name=migration_name, action=self.get_migration_action())
                    if self.get_migration_action() == 'down':
                        MigrationModel.delete().where(MigrationModel.name == migration_name,
                                                      MigrationModel.action == 'up').execute()

            self._save_api_keys()
            self._add_email_templates()
            self._add_roles()
            self._add_admin()
        else:
            print("No new migration collected.")

    def _save_api_keys(self):
        print("\nStarting to generate api key and secret...\n")
        try:
            key = secrets.token_hex(8)
            secret = secrets.token_hex(18)
            app_name = application_settings['application_name'] + ' ' + application_settings["default_module"].capitalize()
            if key is not None and secret is not None:
                name = (list(application_settings['services'].keys())[0])
                apikey = application_settings["services"][name]["api_key"]
                apisecret = application_settings["services"][name]["api_token"]
                result = ApiKeyModel.get_or_none(ApiKeyModel.key == apikey, ApiKeyModel.secret == apisecret)
                if result is None:
                    api_keys: dict = {}
                    api_keys['app_name'] = app_name
                    api_keys['app_description'] = app_name
                    api_keys['key'] = key
                    api_keys["secret"] = secret
                    api_keys["active"] = True
                    api_keys["updated_at"]= datetime.datetime.now()
                    ApiKeyModel.create(**api_keys)
                    print("Please copy api_key and api_token from below and paste in file name dev_(name) or env under services section api_key and api_token. \n")
                    print(f"api_key = {key}")
                    print(f"api_token = {secret}")
                    print("Api keys and token generated successfully.\n")
                else:
                    print("Api keys already exist.")
            else:
                print("Api key and api token is not set in application")
        except Exception as e:
            print(e)

    def _add_email_templates(self):
        from profilelib.libraries.email_templates.model import EmailTemplateModel
        email_templates_directory = self.current_file + '/libraries/email_templates/email_template_files'
        print('Adding email templates..')
        try:
            for files in os.listdir(email_templates_directory):
                if files == '__init__.py':
                    continue
                is_file = os.path.isfile(email_templates_directory + '/' + files)
                if is_file:
                    with open(email_templates_directory + '/' + files) as f:
                        content = f.read()
                        name = os.path.splitext(os.path.basename(files))[0]
                        title = name.replace("_", " ").capitalize()
                        data = {
                            "title": title,
                            "subject": title,
                            "content": content,
                            "type": name,
                            "status": "active",
                            "created_at": datetime.datetime.now(),
                            "updated_at": datetime.datetime.now()
                        }
                        EmailTemplateModel.create(**data)
            print('Added successfully email templates..')
        except Exception as e:
            print(e)

    def _add_roles(self):
        print("\nAdding Roles...")
        admin = {'name':'Administrator','type':'default','slug':'administrator','created_at':datetime.datetime.now(),'updated_at':datetime.datetime.now()}
        RoleModel.create(**admin)
        print(f'Administrator slug = {admin["slug"]}')
        guest = {'name':'Guest','type':'default','slug':'guest','created_at':datetime.datetime.now(),'updated_at':datetime.datetime.now()}
        RoleModel.create(**guest)
        print(f'Guest slug = {guest["slug"]}')

    def _add_admin(self):
        print("\nAdding Admin User...")
        data: dict = {}
        crypt = Crypt()
        data["email"] = 'administrator@admin.com'
        data['email_verified'] = True
        data['name'] = 'Administrator'
        data['password'] = crypt.encrypt('password')
        data["status"] = 'active'
        data["agree_to_terms"] = True
        data['created_at'] = datetime.datetime.now()
        data['updated_at'] = datetime.datetime.now()
        user = UserModel(**data)
        if user.save():
            role = RoleModel.get_or_none(RoleModel.slug == 'administrator')
            if role is not None:
                userRole: dict = {}
                userRole['user_id'] = user.id
                userRole['role_id'] = role.id
                UserRoleModel.create(**userRole)
                print(f'Email = {data["email"]}')
                print(f'Password = {"password"}')
